#include <stdio.h>
#include <math.h>
#include "core.h"
#include "stats.h"


#define EPSILON 0.0000001
#define DELAY_MAX 2*num_chunks+10

int status_print(FILE *f, struct peer *peers, int num_peers, int num_chunks, int t)
{
  int i, j, res;

  if (f) {
    fprintf(f, "Time %d:\n", t);
    for (i = 0; i < num_peers; i++) {
      fprintf(f,"\tPeer %d:", i);
      for (j = 0; j < peers[i].buf_size; j++) {
        if (peers[i].chunks[j]) {
          fprintf(f," %d", peers[i].chunks[j]->chunk);
#if 0
        } else {
          if (t > num_chunks + PO_DELAY)
             printf(" time %d: peer %d misses chunk %d\n", t, i, j);
          res = 0;
#endif
        }
      }
      fprintf(f, "\n");
    }
  }
 
  res = 1;
  for (i = 0; i < num_chunks; i++) {
    if (!comp(i)) res = 0;
  }

  return res;
}

//calculate per chunk delay statistics
//returns true if optimal
int per_chunk_delay_analysis(FILE *stream, struct chunk *chunks, int num_chunks, int num_peers)
{
    int c,p,i;  //chunk, peer
    double d_avg_sum=0;   //average of per chunk delays on each <chunk,node>
    double d_avg_sum_opt=0;   //optimum of d_avg_sum
    double d_max_sum=0;   //average of per chunk max delay on each chunk
    double d_max_sum_opt=0;   //optimum of d_max_sum
    int optimal=1;        // true if optimal
    int d_max_max = 0;
    double dropped = 0;
 
    for (c = 0; c < num_chunks; c++) {
      int d_max=0, d_sum=0;
      double d_avg=0;
#if 0
      int d_dist[DELAY_MAX];
      for (i = 0; i < DELAY_MAX; i++) d_dist[i]=0;
      
      for (p = 0; p < num_peers; p++) {
        int d=0;
        d = peers[p].chunks[c]->completed - c;
        d_max = max(d_max,d);
        d_sum += d;
        if (d >= DELAY_MAX) {
          if (stream) fprintf(stream, "WARNING: Chunk %d arrived to %d with delay %d\n", c, p, d);
        } else { 
          d_dist[d]++;
        }
      }
      //analyze delay distribution
      if (stream) fprintf(stream, "Chunk %d, delay distribution:",c);
      for (i = 0; i <= min(d_max,DELAY_MAX-1); i++) {
        if (stream) fprintf(stream, " %d ", d_dist[i]);
      }
      if (stream) fprintf(stream, "\n");
#endif
      d_avg = chunks[c].avg_delay / chunks[c].received;
      d_max_sum += chunks[c].max_delay;
      d_avg_sum += d_avg;
      d_max_max = max(chunks[c].max_delay, d_max_max);
      dropped += (num_peers - chunks[c].received);
    }

    dropped /= (num_chunks * num_peers);
    d_avg_sum /= num_chunks;
    d_avg_sum_opt = ilogb(num_peers) + (double)((num_peers-(1<<ilogb(num_peers)))*2+1)/num_peers;
    if (d_avg_sum-d_avg_sum_opt > EPSILON) optimal = 0;
    if (stream) fprintf(stream, "Average chunk distribution delay %f (optimal %f)\n", d_avg_sum, d_avg_sum_opt);
    if (stream) fprintf(stream, "Maximum chunk distribution delay %d\n", d_max_max);

    d_max_sum /= num_chunks;
    d_max_sum_opt = PO_DELAY;
    if (d_max_sum-d_max_sum_opt > EPSILON) optimal = 0;
    if (stream) fprintf(stream, "Average chunk diffusion delay %f (optimal %f)\n", d_max_sum, d_max_sum_opt);
    if (!stream) printf("\tTESTresults\t%f\t%f\t%d\n", dropped, d_avg_sum, d_max_max);

    return optimal;
}

//calculate per node delay statistics
void per_node_delay_analysis(FILE *stream, struct chunk *chunks, int num_chunks)
{
#if 0
    int p,c;    //peer, chunk
    
    int d_avg = 0;  //average playout delay
    for (p = 0; p < num_peers; p++) {
      int d = 0;
      for (c = 0; c < num_chunks; c++) {
        if (peers[p].chunks[c]->completed - c > d) d = peers[p].chunks[c]->completed - c;
      }
      //fprintf(trace, "Node %d had max delay of %d \n", p, d);
      d_avg += d;
    }
    //fprintf(trace, "Average playout delay %f\n", (double)d_avg/num_peers);
    if (stream) fprintf(stream,"Average playout delay %f (optimal ?)\n", (double)d_avg/num_peers);
#endif
}
